export default {

    data() {


        let record = {};

        this.index.fields.forEach(field => record[field] = '...');

        return {
            uuid: App.utils.uuid(),
            loading: false,
            uploading: false,
            progress: 0,
            records: [record],
        }
    },

    props: {

        index: {
            type: Object,
            default: null
        },
    },

    mounted() {

    },

    methods: {

        importRecords() {

            this.loading = true;
            this.progress = 0;

            const sendChunksSequentially = (array, size = 50, onProgress) => {

                let  chunks = [];

                for (let i = 0; i < array.length; i += size) {
                    chunks.push(array.slice(i, i + size));
                }

                return chunks.reduce(
                    (promise, chunk, index) => {
                        return promise
                                .then(() => this.$request(`/detektivo/idx/import/${this.index.name}`, {items: chunk}))
                                .then(() => {
                                    if (onProgress) onProgress( Math.round(((index + 1) / chunks.length) * 100) )
                                });
                    },
                    Promise.resolve()
                );
            }

            sendChunksSequentially(this.records, 50, (progress) => this.progress = progress ).then(() => {

                this.loading = false;
                App.ui.notify('Records imported!');

                this.$call('index:imported', true);
                this.$close();

            }).catch(res => {

                this.loading = false;
                App.ui.notify(res.error || 'Importing records failed!', 'error');
            });
        }
    },


    template: /*html*/`
        <div>

            <div class="kiss-size-4">
                <span class="kiss-color-muted">{{ t('Add records') }}:</span> {{ index.label || index.name }}
            </div>

            <div class="kiss-margin" :class="{'kiss-disabled':loading}">
                <label>{{ t('Records') }}</label>
                <field-object v-model="records" :height="auto"></field-object>
            </div>

            <div :class="{'kiss-disabled':loading}">
                <div class="kiss-flex kiss-flex-middle">
                    <div class="kiss-margin-right" v-if="false">
                        <button class="kiss-button kiss-overlay-input" :disabled="uploading">
                            {{ t('Upload file') }}
                            <input type="file" name="file" @change="(e) => {readContents(e.target.file)}" v-if="!uploading" />
                        </button>
                    </div>
                    <div class="kiss-size-xsmall kiss-text-bold" v-if="loading && progress">{{ progress }}%</div>
                    <div class="kiss-flex-1 kiss-margin-right"></div>
                    <div class="kiss-size-xsmall kiss-color-muted kiss-margin-right" v-if="Array.isArray(records) && records.length">
                        {{ records.length }} {{ records.length === 1 ? t('Record') : t('Records') }}
                    </div>
                    <div class="kiss-button-group">
                        <button class="kiss-button kiss-button-primary" :class="{'kiss-disabled':!(Array.isArray(records) && records.length)}" @click="importRecords()">{{ t('Add records') }}</button>
                        <button class="kiss-button" autofocus kiss-dialog-close>{{ t('Close') }}</button>
                    </div>
                </div>
            </div>

        </div>
    `
}
