<?php

use GraphQL\Type\Definition\Type;
use GraphQL\Type\Definition\ObjectType;
use App\GraphQL\Types\JsonType;

$gql->queries['fields']['detektivoSearch'] = [

    'type' => new ObjectType([
        'name'   => 'DetektivoSearchResult',
        'fields' =>  [
            'hits' => Type::listOf(JsonType::instance()),
            'query' => Type::string(),
            'processingTimeMs' => Type::int(),
            'limit' => Type::int(),
            'offset' => Type::int(),
            'estimatedTotalHits' => Type::int(),
        ]
    ]),

    'args' => [

        'index'  => Type::nonNull(Type::string()),
        'limit' => ['type' => Type::int(), 'defaultValue' => 50],
        'offset'  => ['type' => Type::int(), 'defaultValue' => 0],
        'q'  => ['type' => Type::string(), 'defaultValue' => ''],
    ],

    'resolve' => function ($root, $args) use($app) {

        $index = $args['index'];

        if (!$app->module('detektivo')->exists($index)) {
            return [];
        }

        $idx = $app->module('detektivo')->index($index);
        $query = $args['q'];

        if (!trim($query)) {
            $app->response->status = 412;
            return ['error' => 'Query parameter is missing or empty!'];
        }

        return $idx->search($query, [
            'limit' => $args['limit'],
            'offset' => $args['offset'],
        ]);
    }
];
