

export function debounce(func, wait, immediate) {

    let timeout;

    return function () {
        let context = this, args = arguments;
        let later = () => {
            timeout = null;
            if (!immediate) func.apply(context, args);
        };
        let callNow = immediate && !timeout;
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
        if (callNow) func.apply(context, args);
    };
};

export function isInViewport(element) {
    const rect = element.getBoundingClientRect();
    return (
        rect.top >= 0 &&
        rect.left >= 0 &&
        rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
        rect.right <= (window.innerWidth || document.documentElement.clientWidth)
    );
}

export function isElementOnTop(element) {
    const rect = element.getBoundingClientRect();
    let topElement = document.elementFromPoint(rect.left + rect.width / 2, rect.top + rect.height / 2);
    while (topElement && topElement.parentElement) {
        if (topElement === element) {
            return true;
        }
        topElement = topElement.parentElement;
    }
    return false;
}
